// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.database.ICdmDataSource;

/**
 * @author n.hoffmann
 * @created 06.07.2009
 * @version 1.0
 */
public class ChooseDatasourceDialog extends Dialog {


	private static final Logger logger = Logger
			.getLogger(ChooseDatasourceDialog.class);
	private Combo combo_dataSources;
	private int selection;
	private String title;
	private String message;
	private List<ICdmDataSource> dataSources;
	private Button okButton;
	
	/**
	 * @param parentShell
	 */
	public ChooseDatasourceDialog(Shell parentShell) {
		super(parentShell);
		title = "Choose Datasource";
		message = "Choose a data source to connect to. Cancel will close the editor again.";
		
		dataSources = CdmDataSourceRepository.getDefault().getAll();	
		
		selection = -1;
		String lastUsedDataSourceName = CdmDataSourceRepository.getDefault().getLastUsedDataSourceName();
		for(ICdmDataSource dataSource : dataSources){
			selection++;
			if(dataSource.getName().equals(lastUsedDataSourceName)){
				break;
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		
        // create composite
        Composite composite = (Composite) super.createDialogArea(parent);
        // create message
        if (message != null) {
            Label label = new Label(composite, SWT.WRAP);
            label.setText(message);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_CENTER);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            label.setLayoutData(data);
            label.setFont(parent.getFont());
        }
        
		combo_dataSources = new Combo(composite, SWT.DROP_DOWN);
		combo_dataSources.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
		int i = 0;
		
		for (ICdmDataSource dataSource : dataSources){
			combo_dataSources.add(dataSource.getName() + " - " + dataSource.getServer() + ", " + dataSource.getDatabase() , i++);
		}
		
		combo_dataSources.addSelectionListener(new SelectionAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				selection = combo_dataSources.getSelectionIndex();
			}			
		});
		
		combo_dataSources.select(selection);
		
        applyDialogFont(composite);
        return composite;
	}

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
     */
    protected void createButtonsForButtonBar(Composite parent) {
        // create OK and Cancel buttons by default
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
        //do this here because setting the text will set enablement on the ok
        // button
        combo_dataSources.setFocus();
    }
	
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        if (title != null) {
			shell.setText(title);
		}
    }
    
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	@Override
	protected void okPressed() {
		super.okPressed();
		ICdmDataSource chosenDataSource = dataSources.get(selection);		
		CdmDataSourceRepository.getDefault().setCurrentDataSource(chosenDataSource);
	}
}
