// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource.wizard;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;

/**
 * @author n.hoffmann
 * @created 18.05.2009
 * @version 1.0
 */
public class CdmDataSourceWizard extends Wizard {
	
	private CdmDataSourceCredentialsWizardPage dataSourcePage;

	private CdmDataSourceTypeSelectionWizardPage dataSourceSelectionPage;
	
	private ICdmDataSource dataSource;
	
	/**
	 * @param b
	 */
	public CdmDataSourceWizard(ICdmDataSource dataSource) {
		super();
		setForcePreviousAndNextButtons(true);
		setWindowTitle("Datasource Dialog");
		if(dataSource != null){
			this.dataSource = dataSource;
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {

		if(dataSource == null){
			dataSourceSelectionPage = new CdmDataSourceTypeSelectionWizardPage(dataSource);	
			this.addPage(dataSourceSelectionPage);
		}else{
			if(dataSource.getDatabaseType() == DatabaseTypeEnum.H2){
				dataSourcePage = new CdmDataSourceH2WizardPage(dataSource);
			}else if(dataSource.getDatabaseType() == DatabaseTypeEnum.MySQL){
				dataSourcePage = new CdmDataSourceMySQLWizardPage(dataSource);
			}else if(dataSource.getDatabaseType() == DatabaseTypeEnum.SqlServer2005){
				dataSourcePage = new CdmDataSourceSQLServerWizardPage(dataSource);
			}
			this.addPage(dataSourcePage);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		
		String dataSourceName = null;
		ICdmDataSource dataSource = null;
		
		// TODO this is a little bit confusing because we do edit and create in the same wizard.
		// maybe we want to refactor it into separate wizards
		if(dataSourceSelectionPage != null){
			dataSourceName = dataSourceSelectionPage.getDataSourceName();
			if(dataSourceSelectionPage.getCredentialsWizardPage() != null){
				dataSource = dataSourceSelectionPage.getCredentialsWizardPage().getDataSource();
			}else{
				throw new IllegalStateException("Expected a datasource credentials page to exist");
			}
		}else if(dataSourcePage != null){
			dataSource = dataSourcePage.getDataSource();
			dataSourceName = dataSource.getName();
		}else{
			throw new IllegalStateException("Other pages expected");
		}
					
		CdmDataSourceRepository.getDefault().save(dataSourceName, dataSource);
		
		return true;
	}
}
