// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.io.berlinModel.in.BerlinModelImportConfigurator;
import eu.etaxonomy.cdm.io.common.Source;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceSQLServerWizardPage;
import eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceTypeSelectionWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 24.06.2009
 * @version 1.0
 */
public class BerlinModelImportWizard extends AbstractImportWizard<BerlinModelImportConfigurator> {
	private static final Logger logger = Logger
			.getLogger(BerlinModelImportWizard.class);

	CdmDataSourceSQLServerWizardPage dataSourcePage;
	
	BerlinModelImportConfigurator configurator;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		configurator = BerlinModelImportConfigurator.NewInstance(null, null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		ICdmDataSource dataSource =null;
		if(dataSourcePage != null){
			dataSource = dataSourcePage.getDataSource();
		}else{
			throw new IllegalStateException("Expected a datasource credentials page to exist");
		}
		
		// save the new dataSource to repository
//		CdmDataSourceRepository.getDefault().save(dataSourceName, dataSource);
		
		Source source = new Source(Source.SQL_SERVER_2005, 
				dataSource.getServer(),
				dataSource.getDatabase(),
				dataSource.getPort());
		source.setUserAndPwd(dataSource.getUsername(), dataSource.getPassword());
		
		
		configurator.setSource(source);
		
		// should be obsolete because we want to pass application context but is not at the moment (throws NPE when not given)
		configurator.setDestination(CdmDataSourceRepository.getDefault().getCurrentDataSource());
		
		configurator.setCdmAppController(CdmStore.getApplicationController());
		
		return doImportInternal(configurator);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.AbstractImportWizard#getConfigurator()
	 */
	@Override
	public BerlinModelImportConfigurator getConfigurator() {
		return configurator;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.AbstractImportWizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		dataSourcePage = new CdmDataSourceSQLServerWizardPage(null);
		addPage(dataSourcePage);
	}
}
