// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemeManager;

import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 11.05.2009
 * @version 1.0
 */
public abstract class AbstractUtility {
	private static final Logger logger = Logger.getLogger(AbstractUtility.class);
	
	protected static IStatusLineManager statusLineManager;
	
	/**
	 * @return
	 */
	public static Shell getShell() {
		return TaxeditorStorePlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}
	
	public static IWorkbenchPage getActivePage(){
		return TaxeditorStorePlugin.getDefault().getWorkbench()
			.getActiveWorkbenchWindow().getActivePage();
	}
	
	public static boolean closeAll() {
		return getActivePage().closeAllEditors(true);
	}

	public static ITheme getCurrentTheme(){
		IThemeManager themeManager = TaxeditorStorePlugin.getDefault().getWorkbench().getThemeManager();
		return themeManager.getCurrentTheme();
	}
	
	/**
	 * Fonts registered to the plugin may be obtained with the Eclipse themeing functionality.
	 * Thus fonts are chooseable by the user via Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the FontRegistry for the current theme
	 */
	public static FontRegistry getFontRegistry(){
		return getCurrentTheme().getFontRegistry();
	}
	
	/**
	 * Color registered to the plugin may be obtained with the Eclipse themeing functionality.
	 * Thus colors are editable by the user via Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the ColorRegistry for the current theme
	 */
	public static ColorRegistry getColorRegistry(){
		return getCurrentTheme().getColorRegistry();
	}
	
	/**
	 * Open a message box that informs the user about unimplemented functionality.
	 * This method is for developer convenience.
	 */
	public static void notImplementedMessage(){
		warningDialog("Not yet implemented");
	}
	
	public static void warningDialog(String warning){
		MessageBox messageBox = new MessageBox(getShell(), SWT.ICON_WARNING | SWT.OK);
		messageBox.setText(warning);
		messageBox.open();
	}
	
	public static IStatus executeOperation(IUndoableOperation operation){
		if(getOperationHistory() == null){
			throw new IllegalArgumentException("Ther is no operation history for this context");
		}
		
		try {
			IStatus status = getOperationHistory().execute(operation, getMonitor(),
								WorkspaceUndoUtil.getUIInfoAdapter(getShell()));
			String statusString = status.equals(Status.OK_STATUS) ? "completed" : "cancelled";
			setStatusLine(operation.getLabel() + " " + statusString + ".");
			return status;
		} catch (ExecutionException e) {
			logger.error("Error executing operation: " + operation.getLabel(), e);
		}
		return null;
	}
	
	public static IOperationHistory getOperationHistory(){
		return TaxeditorStorePlugin.getDefault().getWorkbench().
							getOperationSupport().getOperationHistory();
	}
	
	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}
	
	protected static IProgressMonitor getMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}
}
