/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class ImageResources {
	
	private static ImageResources instance;
	
	public static ImageResources getInstance(){
		if(instance == null){
			instance = new ImageResources();
		}
		return instance;
	}
	
	
	/* ***************************************************************************************
	ICONS
	************************************************************************************** */
	public static final String EDIT_ICON = "edit_icon";
	public static final String WARNING_ICON = "warning_icon";
	public static final String BLACK_SQUARE_ICON = "black_square_icon";
	public static final String HOMOTYPIC_SYN_ICON = "homotypic_syn_icon";
	public static final String HOMOTYPIC_SYN_ORIGINAL_ICON = "homotypic_syn_original_icon";
	public static final String HETEROTYPIC_SYN_ICON = "heterotypic_syn_icon";
	public static final String HETEROTYPIC_SYN_ORIGINAL_ICON = "heterotypic_syn_original_icon";
	public static final String MISAPPLIED_NAME_ICON = "misapplied_icon";
	public static final String CONCEPT_ICON = "concept_icon";
	public static final String AUTONYM_ICON = "autonym_icon";
	public static final String BASIONYM_ICON = "basionym_icon";
	public static final String ORTHOGRAPHIC_VARIANT_ICON = "orthographic_variants_icon";
	public static final String DB_ICON = "db_icon";
	public static final String MOVE_ICON = "move_icon";
	public static final String ACTIVE_DELETE_ICON = "active_delete_icon";
	public static final String SYNONYM_TO_TAXON_ICON = "synoynm_to_taxon_icon";
	public static final String OPEN_TAXON_ICON = "open_taxon_icon";
	public static final String ADD_CHILD_TAXON_ICON = "add_taxon_icon";
	public static final String SWAP_SYNONYM_AND_TAXON_ICON = "swap_syn_and_tax_icon";
	public static final String QUICK_ADD_ICON = "quick_add_icon";
	public static final String TAXON_TO_SYNONYM_ICON = "tax_to_syn_icon";
	public static final String ERROR_ANNOTATION_ICON = "error_annotation_icon";
	public static final String EDIT_BITMAP_ICON = "edit_bitmap_icon";
	public static final String IMG_DATASOURCE_CONNECTED = "IMG_DATASOURCE_CONNECTED";
	public static final String IMG_DATASOURCE_DISCONNECTED = "IMG_DATASOURCE_DISCONNECTED";

	
	/***************************************************************************
	 * IMAGE REGISTRY
	 **************************************************************************/
	private ImageRegistry imageRegistry;
	
	public static ImageDescriptor getImageDescriptor(String key) {
		return getInstance().getImageRegistry().getDescriptor(key);
	}

	public static Image getImage(String key) {
		return getInstance().getImageRegistry().get(key);
	}
	
	/**
	 * Gets an image for the given URL. Adds the image to the registry as needed.
	 * 
	 * @param url
	 * @return
	 */
	public static Image getImage(URI uri) {		
		
		String key = uri.toString();
		ImageRegistry registry = getInstance().getImageRegistry();

		Image image = registry.get(key);
		
		if (image == null) {
			ImageDescriptor desc;
			try {
				desc = ImageDescriptor.createFromURL(new URL(key));
				registry.put(key, desc);
				image = registry.get(key);
			} catch (MalformedURLException e) {
				// should never happen
			}catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
//			logger.warn(ImageDescriptor.getMissingImageDescriptor().getClass());
//			logger.warn(desc.getClass());
//			
//			if (ImageDescriptor.getMissingImageDescriptor().equals(desc)) {
//				logger.warn("img not found");
//			}
			
		}
		
		return image;
	}
	
    public ImageRegistry getImageRegistry() {
        if (imageRegistry == null) {
            imageRegistry = TaxeditorStorePlugin.getDefault().createImageRegistry();
            initializeImageRegistry(imageRegistry);
        }
        return imageRegistry;
    }

	protected void initializeImageRegistry(ImageRegistry registry) {
		registerImage(registry, EDIT_ICON, "edit_16x16.ico");
		registerImage(registry, WARNING_ICON,
				"warn_tsk.gif");
		registerImage(registry, BLACK_SQUARE_ICON,
				"accepted_small.gif");
		registerImage(registry, HOMOTYPIC_SYN_ICON,
				"homosyn_no_bg.gif");
		registerImage(registry,
				HOMOTYPIC_SYN_ORIGINAL_ICON,
				"homosyn_original_no_bg.gif");
		registerImage(registry, HETEROTYPIC_SYN_ICON,
				"heterosyn_no_bg.gif");
		registerImage(registry,
				HETEROTYPIC_SYN_ORIGINAL_ICON,
				"heterosyn_original_no_bg.gif");
		registerImage(registry, MISAPPLIED_NAME_ICON,
				"misapplied_no_bg.gif");
		registerImage(registry, CONCEPT_ICON,
				"concept_no_bg.gif");
		registerImage(registry, AUTONYM_ICON,
				"autonym_no_bg.gif");
		registerImage(registry, BASIONYM_ICON,
				"basionym_no_bg.gif");
		registerImage(registry, ORTHOGRAPHIC_VARIANT_ICON,
				"orthovariant_no_bg.gif");
		registerImage(registry, DB_ICON, "db.gif");
		registerImage(registry, MOVE_ICON,
				"correction_change.gif");
		registerImage(registry, ACTIVE_DELETE_ICON,
				"delete_edit.gif");
		registerImage(registry, SYNONYM_TO_TAXON_ICON,
				"change.gif");
		registerImage(registry, OPEN_TAXON_ICON, "open.gif");
		registerImage(registry, ADD_CHILD_TAXON_ICON,
				"new_child.gif");
		registerImage(registry,
				SWAP_SYNONYM_AND_TAXON_ICON, "swap2.gif");
		registerImage(registry, QUICK_ADD_ICON,
				"quick_add.gif");
		registerImage(registry, TAXON_TO_SYNONYM_ICON,
				"tax_to_syn.gif");
		registerImage(registry, ERROR_ANNOTATION_ICON,
				"error_co.gif");
		registerImage(registry, EDIT_BITMAP_ICON,
				"256color_16x16.bmp");
		registerImage(registry, IMG_DATASOURCE_CONNECTED, 
				"prj_obj.gif");
		registerImage(registry, IMG_DATASOURCE_DISCONNECTED, 
		"cprj_obj.gif");
	}

	private void registerImage(ImageRegistry registry, String key,
			String fileName) {
		try {
			IPath path = new Path("icons/" + fileName); //$NON-NLS-1$
			URL url = FileLocator.find(TaxeditorStorePlugin.getDefault().getBundle(), path, null);
			if (url != null) {
				ImageDescriptor desc = ImageDescriptor.createFromURL(url);
				registry.put(key, desc);
			}
		} catch (Exception e) {
		}
	}
}
