/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * @author p.ciardelli
 * @created 14.01.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class ChangeSynonymToMisapplicationOperation extends AbstractPostOperation {

	private Synonym synonym;
	private Taxon misapplication;
	private Set<SynonymRelationshipType> synonymTypes;

	public ChangeSynonymToMisapplicationOperation(String label,
			IUndoContext undoContext, Taxon taxon, Synonym synonym, IPostOperationEnabled editor) {
		super(label, undoContext, taxon, editor);
		
		this.synonym = synonym;
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// get name from synonym
		TaxonNameBase<?, ?> synonymName = synonym.getName();
		
		// make misapplied name with synonym name
		misapplication = Taxon.NewInstance(synonymName, null);
		
		// store synonymRelationshipType for later undo operations
		synonymTypes = synonym.getRelationType(taxon);
		
		// remove synonym from taxon		
		taxon.removeSynonym(synonym);
		
		// add misapplied name to taxon
		// TODO add microcitation for misapplied name to property sheet (if microcitation is indeed needed?!)
		taxon.addMisappliedName(misapplication, null, null);	
		
		// redraw editor if exists
		return postExecute(misapplication);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// add misapplied name to taxon
		// TODO add citation for misapplied name to property sheet
		taxon.addMisappliedName(misapplication, null, null);
		
		// remove synonym from taxon
		taxon.removeSynonym(synonym);
		
		// redraw editor if exists
		return postExecute(null);
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// remove misapplied name from taxon
		taxon.removeTaxon(misapplication, TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		
		// add synonym to taxon
		for (SynonymRelationshipType synonymType : synonymTypes){
			taxon.addSynonym(synonym, synonymType);
		}
		
		// redraw editor if exists
		return postExecute(null);
	}
}
