/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.parser.CdmParserUtil;

/**
 * <ol>
 * 	<li>Create synonym name using newSynonymText</li>
 * 	<li>Add name to new homotypic group</li>
 * 	<li>Create a new synonym for the taxon</li>
 * </ol>
 * @author p.ciardelli
 * @created 16.01.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class CreateSynonymInNewGroupOperation extends AbstractPostOperation {
	
	private String newSynonymText;
	private SynonymRelationship newSynonymRelationship;

	public CreateSynonymInNewGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, String newSynonymText, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		this.newSynonymText = newSynonymText;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Create synonym name using newSynonymText
		TaxonNameBase<?, ?> synonymName = 
				CdmParserUtil.parseFullReference(newSynonymText, null, null);
		
		// Add name to new homotypic group
		HomotypicalGroup.NewInstance().addTypifiedName(synonymName);
		
		// Create a new synonym for the taxon
		newSynonymRelationship = taxon.addHeterotypicSynonymName(synonymName);
				
		return postExecute(newSynonymRelationship.getSynonym());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Remove the synonym 
		taxon.removeSynonymRelation(newSynonymRelationship);
		newSynonymRelationship = null;
		
		return postExecute(null);
	}
}
