/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.parser;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.strategy.parser.INonViralNameParser;
import eu.etaxonomy.cdm.strategy.parser.NonViralNameParserImpl;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 22.12.2008
 * @version 1.0
 */
public class CdmParserUtil {
	private static final Logger logger = Logger
			.getLogger(CdmParserUtil.class);
	
	private static INonViralNameParser nonViralNameParser;
	
	private static INonViralNameParser getNonViralNameParser() {
		if (nonViralNameParser == null) {
			nonViralNameParser = NonViralNameParserImpl.NewInstance();
		}
		return nonViralNameParser;
	}
	
	public static void clearNonViralNameParser() {
		nonViralNameParser = null;
	}
	
	/**
	 * Takes a raw string, returns a brand spanking new name object.
	 * <p>
	 * If no nomenclatural code is specified, the code specified in user preferences
	 * is used, i.e. if the preferred code is ICBN, a <code>BotanicalName</code> is returned; 
	 * if it's ICZN, a <code>ZoologicalName</code> is returned.
	 * </p>
	 * 
	 * @param fullReference
	 * @param nomCode
	 * @param rank
	 * @return
	 */
	public static TaxonNameBase parseFullReference(String fullReference,
			NomenclaturalCode nomCode, Rank rank) {
		
		if (nomCode == null) {
			nomCode = PreferencesUtil.getPreferredNomenclaturalCode();
		}
		
		TaxonNameBase name = getNonViralNameParser().parseReferencedName(fullReference,
				nomCode, rank);

		if (name.hasProblem()) {
			name.setFullTitleCache(fullReference);
		}
		
		return name;
	}

	/**
	 * Takes an existing name object and a string, returns the same name
	 * object with the fields parsed from the string.
	 * 
	 * @param nameToBeFilled
	 * @param fullReference
	 * @param rank
	 * @param makeEmpty
	 */
	public static void parseFullReference(NonViralName nameToBeFilled, String fullReference) {
		
		Rank rank = null;
		boolean makeEmpty = true;
		
		// If the name already has a rank, make sure it is passed to the parser
		if (nameToBeFilled.getRank() != null) {
			rank = nameToBeFilled.getRank();
		}
						
		getNonViralNameParser().parseReferencedName(nameToBeFilled, fullReference, rank, makeEmpty);
		
		if (nameToBeFilled.hasProblem()) {
			nameToBeFilled.setFullTitleCache(fullReference);
		}
	}
	
	/**
	 * @param nameToBeFilled
	 * @param fullNameString
	 * @param rank
	 * @param makeEmpty
	 */
	public static void parseFullName(NonViralName nameToBeFilled,
			String fullNameString, Rank rank, boolean makeEmpty) {
		getNonViralNameParser().parseFullName(nameToBeFilled, fullNameString,
				rank, makeEmpty);
	}
	
}
