// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.user.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.model.CommonHelper;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 02.07.2009
 * @version 1.0
 */
public class UserWizardPage extends WizardPage implements ModifyListener{
	private Composite composite;
	private Text text_username;
	private Text text_oldPassword;
	private Text text_password;
	private Text text_passwordRepeat;
	private User user;
	private boolean newMode = false;
	private Group group_person;
	private Text text_personFirstName;
	private Text text_personLastName;
	private Text text_userEmail;

	/**
	 * @param pageName
	 */
	protected UserWizardPage(User user) {
		super("USER_WIZARD_PAGE");
		
		this.user = user;
		
		if(user == null){
			setTitle("Create new user");
			setDescription("Enter information for new user.");
			newMode = true;
			this.user = User.NewInstance(null, null);
		}else{
			setTitle("Edit existing user");
			setDescription("Edit information for the selected user.");
			
		}		
	}

	private static final Logger logger = Logger.getLogger(UserWizardPage.class);

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		// right - edit Form
		composite = new Composite(parent, SWT.NULL);
		GridLayout compositeLayout = new GridLayout();
		compositeLayout.numColumns = 2;
		composite.setLayout(compositeLayout);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		CLabel label_username = new CLabel(composite, SWT.NULL);
		label_username.setText("Username");
		text_username = new Text(composite, SWT.BORDER);
		text_username.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_username.addModifyListener(this);
		
		if(! newMode){
			text_username.setEditable(false);
			text_username.setBackground(Resources.getColor(Resources.SEARCH_VIEW_FOREGROUND));
		}
		
		if(newMode){
			createPasswordFieldsNew(composite);
		}else if(CdmStore.getLoginManager().getAuthenticatedUser().equals(user)){
			createPasswordFieldsEdit(composite);
		}

		group_person = createPersonGroup(composite);
		group_person.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		
		setControl(composite);
		
		init();
	}

	private void createPasswordFieldsEdit(Composite parent){
		CLabel label_oldPassword = new CLabel(parent, SWT.NULL);
		label_oldPassword.setText("Old Password");
		text_oldPassword = new Text(parent, SWT.BORDER);
		text_oldPassword.setEchoChar('*');
		text_oldPassword.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_oldPassword.addModifyListener(this);
		
		createPasswordFieldsNew(parent);
	}
	
	private void createPasswordFieldsNew(Composite parent){
		CLabel label_password = new CLabel(parent, SWT.NULL);
		label_password.setText("Password");
		
		text_password = new Text(parent, SWT.BORDER);
		text_password.setEchoChar('*');
		text_password.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_password.addModifyListener(this);
		
		new CLabel(parent, SWT.NULL);
		text_passwordRepeat = new Text(parent, SWT.BORDER);
		text_passwordRepeat.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_passwordRepeat.setEchoChar('*');
		text_passwordRepeat.addModifyListener(this);
	}
	
	private Group createPersonGroup(Composite parent){

		Group group = new Group(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		group.setLayout(layout);
		
		CLabel label_personFirstName = new CLabel(group, SWT.NULL);
		label_personFirstName.setText("First Name");
		text_personFirstName = new Text(group, SWT.BORDER);
		text_personFirstName.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		text_personFirstName.addModifyListener(this);
		
		CLabel label_personLastName = new CLabel(group, SWT.NULL);
		label_personLastName.setText("Last Name");
		text_personLastName = new Text(group, SWT.BORDER);
		text_personLastName.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		text_personLastName.addModifyListener(this);
		
		
		CLabel label_userEmail = new CLabel(group, SWT.NULL);
		label_userEmail.setText("Email");
		text_userEmail = new Text(group, SWT.BORDER);
		text_userEmail.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		text_userEmail.addModifyListener(this);
		
		return group;
	}
	
	/**
	 * 
	 */
	private void init() {
		CommonHelper.editTextWithoutTriggeringListeners(text_username, this, user.getUsername());
		if(user.getPerson() == null){
			user.setPerson(Person.NewInstance());
		}else{	
			CommonHelper.editTextWithoutTriggeringListeners(text_personFirstName, this, user.getPerson().getFirstname());
			CommonHelper.editTextWithoutTriggeringListeners(text_personLastName, this, user.getPerson().getLastname());
			CommonHelper.editTextWithoutTriggeringListeners(text_userEmail, this, user.getEmailAddress());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		if(newMode){
			user.setUsername(text_username.getText().trim());
			user.setPassword(text_password.getText());
		}
		user.getPerson().setFirstname(text_personFirstName.getText());
		user.getPerson().setLastname(text_personLastName.getText());
		user.setEmailAddress(text_userEmail.getText());
	}

	/**
	 * @return
	 */
	public User getUser() {
		return user;
	}
}
